"""
Simple script to print the right side of dependencies as filenames.
"""

from os.path import join
from snakefood.filter import do_filter

for _, (tbase, tfn) in do_filter():
    if tbase is not None:
        print join(tbase, tfn)

